EipJsonTransformXml.prototype._ICPDASEIPDri_ = async function() {	
    const Class = new LoadFile();

    try {
        const [
            EipModule, EipSetting, EipVariable, EipConnectionParameters, EipProductSpecification
        ] = await Promise.all([
            Class.ReadFile("./File/EIP_Module/Module.json", {}),
            Class.ReadFile("./File/EIP_Module/EipSetting.json", {}),
            Class.ReadFile("./File/EIP_Module/EipVariable.json", {}),
            Class.ReadFile("./File/EIP_Module/ConnectionParameters.json", {}),
            Class.ReadFile("./File/EIP_Module/ProductSpecification.json", {})
        ]);

        const IndexKey = Object.keys(EipModule["LAN"]);
        const EipDriData = {
            EtherNetIPDriver: {
                EIPTask: []
            }
        };

        for (let i = 0; i < IndexKey.length; i++) {
            const ModuleName = EipModule["LAN"][IndexKey[i]]["ModuleName"];
            EipDriData["EtherNetIPDriver"]["EIPTask"][i] = {
                AssemblyData: {}
            };

            const EipDriAssemblyData = EipDriData["EtherNetIPDriver"]["EIPTask"][i]["AssemblyData"];
            EipJsonTransformXml.prototype._GetEipDriAssemblyData_(EipConnectionParameters, EipDriAssemblyData, ModuleName);

            const EipDriTaskData = EipDriData["EtherNetIPDriver"]["EIPTask"][i];
            EipJsonTransformXml.prototype._GetEipDriTaskData_(EipDriTaskData, EipSetting, IndexKey[i], ModuleName);

            EipDriData["EtherNetIPDriver"]["EIPTask"][i]["EIPItem"] = [];
            const ChannelTick = parseInt(EipSetting[IndexKey[i]]["ModeType"].split("-")[0]);

            for (let j = 0; j < ChannelTick; j++) {
                const EipVariableKey = Object.keys(EipVariable[IndexKey[i]]);
                const TypeString = EipVariableKey[j].split("-")[0];
                const moduleId = parseInt(EipModule["LAN"][IndexKey[i]]["ModuleName"].split("-")[1]);
                const I_O_Property = EipJsonTransformXml.prototype._GetIOPropertyIndex_(TypeString);
                const channel = EipVariableKey[j].split("-")[2];
                const NickName = EipSetting[IndexKey[i]]["NickName"];

                EipDriData["EtherNetIPDriver"]["EIPTask"][i]["EIPItem"][j] = {
                    Name: `EIP_No.${IndexKey[i]}_${NickName}_${TypeString}.${EipVariable[IndexKey[i]][EipVariableKey[j]]["VariableName"]}`,
                    Property: I_O_Property,
                    IsReverse: 0,
                    InputByteOffset: EipJsonTransformXml.prototype._GetInputOffset_(moduleId, I_O_Property, channel),
                    OutputByteOffset: EipJsonTransformXml.prototype._GetOutputOffset_(moduleId, I_O_Property, channel)
                };
            }
        }

        const sortedEipDriData = EipJsonTransformXml.prototype._EipSortTaskId_(EipDriData);
        const xotree = new XML.ObjTree();
        const xml = xotree.writeXML(sortedEipDriData);
        return xml;
    } catch (error) {
        //console.error(`Error reading files: ${error}`);
        return "";
    }
};


EipJsonTransformXml.prototype._GetEipDriAssemblyData_ = function(EipConnectionParameters, EipDriAssemblyData, ModuleName)
{
	//EipDriAssemblyData = new Object();
	EipDriAssemblyData["InputAssembly"] = EipConnectionParameters[ModuleName]["Assembly"]["Input"];
	EipDriAssemblyData["InputByteSize"] = EipConnectionParameters[ModuleName]["Size"]["Input"];
	EipDriAssemblyData["OutputAssembly"] = EipConnectionParameters[ModuleName]["Assembly"]["Output"];
	EipDriAssemblyData["OutputByteSize"] = EipConnectionParameters[ModuleName]["Size"]["Output"];
}		
EipJsonTransformXml.prototype._GetEipDriTaskData_ = function(EipDriTaskData, EipSetting, number, ModuleName)
{		
	EipDriTaskData["IP"] = EipSetting[number]["IP"];
	EipDriTaskData["PollingRate"] = 100;
	EipDriTaskData["TaskID"] = -1;
	EipDriTaskData["TaskName"] = EipSetting[number]["NickName"];
	EipDriTaskData["ModuleName"] = ModuleName.split("-")[1];
}
EipJsonTransformXml.prototype._GetIOPropertyIndex_ = function(string)
{
	//var I_O = { "AI": 1, "AO": 2, "DI": 3, "DO":4,"DICounter":5,"Counter":6}
	if(string=='AI')
		return 1;
	else if(string=='AO')
		return 2;
	else if(string=='DI')
		return 3;
	else if(string=='DO')
		return 4;
}

EipJsonTransformXml.prototype._GetInputOffset_ = function(moduleId, I_O_Property, channel)
{
	var Segment = "";

    switch (I_O_Property) {
        case 1:
            switch (moduleId) {
                case 2017:
                case 2019:
                    Segment = channel*2;
                    break;
            }
            break;
        case 2:
            break;
        case 3:
            switch (moduleId) {
                case 2051:
                case 2055:
                case 2060:
                    Segment = parseInt(channel / 8) + "@" + channel % 8;
                    break;
            }
            break;
        case 4:
            switch (moduleId) {
                case 2042:
                    Segment = parseInt(channel / 8) + "@" + channel % 8;
                    break;
                case 2055:
                case 2060:
                    Segment = 1+parseInt(channel / 8) + "@" + channel % 8;
                    break;
            }
            break;
        case 5:
            switch (moduleId) {
                case 2051:
                case 2060:
                    Segment = 2 + channel * 4;
                    break;
            }
            break;
        default:
            break;
    }
    return Segment;
}

EipJsonTransformXml.prototype._GetOutputOffset_ = function(moduleId, I_O_Property, channel)
{
	var Segment = "";

    switch (I_O_Property)
    {
        case 4:
            switch (moduleId)
            {
                case 2042:
                case 2055:
                case 2060:
                    Segment = parseInt(channel / 8) + "@" + channel % 8;
                    break;
            }
            break;
        default:
            break;
    }
    return Segment;
}

EipJsonTransformXml.prototype._EipSortTaskId_ = function(object) {
    var length = object["EtherNetIPDriver"]["EIPTask"].length;
    for (var i = 0; i < length; i++) {
        object["EtherNetIPDriver"]["EIPTask"][i]["TaskID"] = i;
    }
    return object;
}