ControllerServiceSetting.prototype._SaveDeviceDri_ = async function(Result) 
{
    const Class = new JsonTransformXml();
    
    const [
        ModbusDri,
        MqttDri,
        PidDriverDri,
        XVDri,
        EipDir
    ] = await Promise.all([
        Class.TransformModbusDri(),
        Class.TransformMqttDri(),
        Class.TransformPidDriverDri(),
        Class.TransformXVBoardDri(),
        (new EipJsonTransformXml()).ICPDASEIPDri()
    ]);
    
    const TransmissionClass = new Transmission();

    const wrapInPromise = (method, ...args) => {
        return new Promise((resolve) => {
            const callback = (result) => {
                resolve(result);
            };
            method(...args, callback);
        });
    };

    const results = await Promise.all([
        wrapInPromise(TransmissionClass.TransmissionString, "SaveModbusDri", ModbusDri, {}),
        wrapInPromise(TransmissionClass.TransmissionString, "SaveMqttDri", MqttDri, {}),
        wrapInPromise(TransmissionClass.TransmissionString, "SavePidDriverDri", PidDriverDri, {}),
        wrapInPromise(TransmissionClass.TransmissionString, "SaveEipDri", EipDir, {}),
        wrapInPromise(TransmissionClass.TransmissionString, "SaveXVDri", XVDri, {})
    ]);

    if (results.every(result => result.Input === true)) {
        Result.Input = true;
    }
};


ControllerServiceSetting.prototype._SaveServiceXml_ = async function(Result) 
{
    const Class = new JsonTransformXml();
	const ModuleClass = new ModuleToLocalDataLoggerCommon();

    const [
        VariableXml,
        UAServerDataXml,
        MqttServiceXml,
        MethodTableXml,
        ScaleVariableTableXml,
        BitwiseVariableTableXml,
        UAServerConfigXml
    ] = await Promise.all([
        Class.TransformVariableXml(),
        Class.TransformUAServerDataXml(),
        Class.TransformMqttServiceXml(),
        Class.TransformMethodTableXml(),
        Class.TransformScaleVariableTableXml(),
        Class.TransformBitwiseVariableTableXml(),
        Class.TransformUAServerConfigXml()
    ]);

    const DataLogServiceXml = await ModuleClass.GetDataLogServiceXml();

    const TransmissionClass = new Transmission();

    const wrapInPromise = (method, ...args) => {
        return new Promise((resolve) => {
            const callback = (result) => {
                resolve(result);
            };
            method(...args, callback);
        });
    };

    const results = await Promise.all([
        wrapInPromise(TransmissionClass.TransmissionString, "SaveVariableTableXml", VariableXml, {}),
        wrapInPromise(TransmissionClass.TransmissionString, "SaveUAServerDataXml", UAServerDataXml, {}),
        wrapInPromise(TransmissionClass.TransmissionString, "SaveMQTTServiceXml", MqttServiceXml, {}),
        wrapInPromise(TransmissionClass.TransmissionString, "SaveMethodTableXml", MethodTableXml, {}),
        wrapInPromise(TransmissionClass.TransmissionString, "SaveDataLogServiceXml", DataLogServiceXml, {}),
        wrapInPromise(TransmissionClass.TransmissionString, "SaveScaleVariableTableXml", ScaleVariableTableXml, {}),
        wrapInPromise(TransmissionClass.TransmissionString, "SaveBitwiseVariableTableXml", BitwiseVariableTableXml, {}),
        wrapInPromise(TransmissionClass.TransmissionArray, "SaveOpcUaLocalServer", UAServerConfigXml, { Input: null })
    ]);

    if (results.every(result => result.Input === true)) {
        Result.Input = true;
    }
    
};


ControllerServiceSetting.prototype._SaveXml_ = async function(that) 
{
    const Result_1 = { "Input": null };
    const Result_2 = { "Input": null };

    await Promise.all([
        ControllerServiceSetting.prototype._SaveDeviceDri_(Result_1),
        ControllerServiceSetting.prototype._SaveServiceXml_(Result_2)
    ]);

    const waitForResults = () => {
        return new Promise((resolve) => {
            const interval = setInterval(() => {
                if (Result_1.Input === true && Result_2.Input === true) {
                    clearInterval(interval);
                    resolve();
                }
            }, 100);
        });
    };

    await waitForResults();
    ControllerServiceSetting.prototype._RunRuntime_(that);
};

ControllerServiceSetting.prototype._RunRuntime_ = async function(that) 
{
	$(that).closest('.Column').find(".Message").eq(1).show();

	const TransmissionClass = new Transmission();

	const wrapInPromise = (method, ...args) => {
        return new Promise((resolve) => {
            const callback = (result) => {
                resolve(result);
            };
            method(...args, callback);
        });
    };

	try {
		const Result_1 = await wrapInPromise(TransmissionClass.TransmissionArray, "StartMainSystem", [], {});
		if (Result_1.Input === true) {
			const Result_2 = await wrapInPromise(TransmissionClass.TransmissionArray, "StartBackstageService", [], {});
			if (Result_2.Input === true) {
				ControllerServiceSetting.prototype._SwitchRadioStart_(that);
				$(that).closest(".Row").find(".Message").hide();
			}
		}
	} catch (error) {
		//console.error('Error during transmission:', error);
	}
};

	

ControllerServiceSetting.prototype._SwitchRadioStop_ = function(that)
{
	var Row=$(that).closest(".Row");
	var Radio=Row.find(":radio");
	Radio.get(1).checked=true;
};

ControllerServiceSetting.prototype._SwitchRadioStart_ = function(that)
{
	var Row=$(that).closest(".Row");
	var Radio=Row.find(":radio");
	Radio.get(0).checked=true;
};
ControllerServiceSetting.prototype._IsUsedStatus_ = function()
{
	var Return=true;
	if($("#Top-Menu-SystemSetting").css("border-bottom-color") != "rgb(57, 93, 139)")
	{
		if($("#Left-List-ControllerServiceSetting").css("border-left-color") != "rgb(57, 93, 139)")
		{
			clearInterval(Global_TimeSetInterval);
			Return=false;
		}
	}
	return Return;
}

ControllerServiceSetting.prototype._GetMainSystemStatus_ = function()
{
	var Row=$("#Main_Center-Content").find(".Form").find(".Row").eq(0);
	var ssocket = io.connect();
	Global_TimeSetInterval = setInterval(function(){
	if(ControllerServiceSetting.prototype._IsUsedStatus_() == false)
	{
		ssocket.emit("SocketClose");
		return 0;
	}
	ssocket.emit('GetMainSystemStatus');
	ssocket.on('GetMainSystemStatus', function (Return) 
	{
		if(Return == true)
		{
			var Radio = Row.eq(0).find(":radio");
			Radio.eq(0).attr("checked", true);
			Radio.get(0).checked=true;
		}
		else if(Return == false)
		{
			var Radio = Row.eq(0).find(":radio");
			Radio.eq(1).attr("checked", true);
			Radio.get(1).checked=true;
		}
		ssocket.emit("SocketClose");
	});
	}, 1000);
};

ControllerServiceSetting.prototype._StartMainSystem_ = function(that)
{
	$(that).closest(".Row").find(".Message").show();
	var ssocket = io.connect();
	ssocket.emit("SaveFinished");
	ssocket.on("SaveFinished", function (Return)
	{
		 ControllerServiceSetting.prototype._SaveXml_(that);
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._StopMainSystem_ = function(that)
{
	var ssocket = io.connect();
	ssocket.emit('StopMainSystem');
	ssocket.on('StopMainSystem', function (Return) {
		if(Return == true)
		{
			ControllerServiceSetting.prototype._SwitchRadioStop_(that);
		}
		else if(Return == false)
		{
		}
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._SwitchMainSystem_ = function(that)
{
	var Row=$(that).closest(".Row");
	var Radio=Row.find(":radio");
	if(Radio.get(0).checked == true)
	{
		Radio.get(0).checked=false;
		ControllerServiceSetting.prototype._StartMainSystem_(that);
	}
	else if(Radio.get(1).checked == true)
	{
		Radio.get(1).checked=false;
		ControllerServiceSetting.prototype._StopMainSystem_(that);
	}
};
ControllerServiceSetting.prototype._SwitchWDT_ = function(that)
{
	var Form = $(".Form");
	var Row = Form.eq(0).find(".Row");
	var WDT = Row.eq(0).find(":checkbox").prop("checked");
	if(WDT==true)
	{
		ControllerServiceSetting.prototype._StartWDT_(WDT);
		//ControllerServiceSetting.prototype._GetMainSystemStatus_();
	}
	else
	{
		ControllerServiceSetting.prototype._StopWDT_(WDT);
	}
	
};
ControllerServiceSetting.prototype._StartWDT_ = function(WDT)
{
	var ssocket = io.connect();
	ssocket.emit('StartWDT');
	ssocket.on('StartWDT', function (Return) {
		ssocket.emit("SocketClose");
	});
}
ControllerServiceSetting.prototype._StopWDT_ = function(WDT)
{
	var ssocket = io.connect();
	ssocket.emit('StopWDT');
	ssocket.on('StopWDT', function (Return) {
		ssocket.emit("SocketClose");
	});
}
ControllerServiceSetting.prototype._GetMqttBrokerStatus_ = function()
{
	var Row=$("#Main_Center-Content").find(".Form").find(".Row").eq(1);
	var ssocket = io.connect();
	ssocket.emit('GetMqttBrokerStatus');
	ssocket.on('GetMqttBrokerStatus', function (Return) {
		if(Return == true)
		{
			Row.eq(0).find(":radio").eq(0).attr("checked", true);
		}
		else if(Return == false)
		{
			Row.eq(0).find(":radio").eq(1).attr("checked", true);
		}
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._StartMqttBroker_ = function(that)
{
	var ssocket = io.connect();
	ssocket.emit('StartMqttBroker');
	ssocket.on('StartMqttBroker', function (Return) {
		if(Return == true)
		{
			ControllerServiceSetting.prototype._SwitchRadioStart_(that);
		}
		else if(Return == false)
		{
		}
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._StopMqttBroker_ = function(that)
{
	var ssocket = io.connect();
	ssocket.emit('StopMqttBroker');
	ssocket.on('StopMqttBroker', function (Return) {
		if(Return == true)
		{
			ControllerServiceSetting.prototype._SwitchRadioStop_(that);
		}
		else if(Return == false)
		{
		}
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._SwitchMqttBroker_ = function(that)
{
	var Row=$(that).closest(".Row");
	var Radio=Row.find(":radio");
	if(Radio.get(0).checked == true)
	{
		Radio.get(0).checked=false;
		ControllerServiceSetting.prototype._StartMqttBroker_(that);
	}
	else if(Radio.get(1).checked == true)
	{
		Radio.get(1).checked=false;
		ControllerServiceSetting.prototype._StopMqttBroker_(that);
	}
};

ControllerServiceSetting.prototype._GetDdnsStatus_ = function()
{
	var Row=$("#Main_Center-Content").find(".Form").find(".Row").eq(2);
	var ssocket = io.connect();
	ssocket.emit('GetDdnsStatus');
	ssocket.on('GetDdnsStatus', function (Return) {
		if(Return == true)
		{
			Row.eq(0).find(":radio").eq(0).attr("checked", true);
		}
		else if(Return == false)
		{
			Row.eq(0).find(":radio").eq(1).attr("checked", true);
		}
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._StartDdns_ = function(that)
{
	var ssocket = io.connect();
	ssocket.emit('StartDdns');
	ssocket.on('StartDdns', function (Return) {
		if(Return == true)
		{
			ControllerServiceSetting.prototype._SwitchRadioStart_(that);
		}
		else if(Return == false)
		{
		}
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._StopDdns_ = function(that)
{
	var ssocket = io.connect();
	ssocket.emit('StopDdns');
	ssocket.on('StopDdns', function (Return) {
		if(Return == true)
		{
			ControllerServiceSetting.prototype._SwitchRadioStop_(that);
		}
		else if(Return == false)
		{
		}
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._SwitchDdns_ = function(that)
{
	var Row=$(that).closest(".Row");
	var Radio=Row.find(":radio");
	if(Radio.get(0).checked == true)
	{
		Radio.get(0).checked=false;
		ControllerServiceSetting.prototype._StartDdns_(that);
	}
	else if(Radio.get(1).checked == true)
	{
		Radio.get(1).checked=false;
		ControllerServiceSetting.prototype._StopDdns_(that);
	}
};

ControllerServiceSetting.prototype._GetSystemLogStatus_ = function()
{
	var Row=$("#Main_Center-Content").find(".Form").find(".Row").eq(3);
	var ssocket = io.connect();
	ssocket.emit('GetSystemLogStatus');
	ssocket.on('GetSystemLogStatus', function (Return) {
		if(Return == true)
		{
			Row.eq(0).find(":radio").eq(0).attr("checked", true);
		}
		else if(Return == false)
		{
			Row.eq(0).find(":radio").eq(1).attr("checked", true);
		}
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._StartSystemLog_ = function(that)
{
	var ssocket = io.connect();
	ssocket.emit('StartSystemLog');
	ssocket.on('StartSystemLog', function (Return) {
		if(Return == true)
		{
			ControllerServiceSetting.prototype._SwitchRadioStart_(that);
		}
		else if(Return == false)
		{
		}
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._StopSystemLog_ = function(that)
{
	var ssocket = io.connect();
	ssocket.emit('StopSystemLog');
	ssocket.on('StopSystemLog', function (Return) {
		if(Return == true)
		{
			ControllerServiceSetting.prototype._SwitchRadioStop_(that);
		}
		else if(Return == false)
		{
		}
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._SwitchSystemLog_ = function(that)
{
	var Row=$(that).closest(".Row");
	var Radio=Row.find(":radio");
	if(Radio.get(0).checked == true)
	{
		Radio.get(0).checked=false;
		ControllerServiceSetting.prototype._StartSystemLog_(that);
	}
	else if(Radio.get(1).checked == true)
	{
		Radio.get(1).checked=false;
		ControllerServiceSetting.prototype._StopSystemLog_(that);
	}
};


ControllerServiceSetting.prototype._GetMainSystemWDTStatus_ = function()
{
	var Row=$("#Main_Center-Content").find(".Form").find(".Row").eq(0);
	var Form = $(".Form");
	var Row = Form.eq(0).find(".Row");
	
	var ssocket = io.connect();
	ssocket.emit('GetMainSystemWDTStatus');
	ssocket.on('GetMainSystemWDTStatus', function (Return) 
	{
		if(Return == true)
		{
			Row.eq(0).find(":checkbox").prop("checked", Return);
			//var a=1+1;
			//Row.eq(0).find(":radio").eq(0).attr("checked", true);
		}
		else if(Return == false)
		{
			Row.eq(0).find(":checkbox").prop("checked", Return);
			//var a=1+2;
			//Row.eq(0).find(":radio").eq(1).attr("checked", true);
		}
		ssocket.emit("SocketClose");
	});
};

ControllerServiceSetting.prototype._ShowUi_ = function()
{
	ControllerServiceSetting.prototype._GetMainSystemStatus_();
	ControllerServiceSetting.prototype._GetMqttBrokerStatus_();
	ControllerServiceSetting.prototype._GetDdnsStatus_();
	ControllerServiceSetting.prototype._GetSystemLogStatus_();
};
